/**************************************************************/
/*                                                            */
/*   File name :  libvssp32.h                                 */
/*                                                            */
/*      Version:   2010-08-03                                 */
/*                                                            */
/*   Copyright (c) 2006-2009 T.Kondo/NICT All Right Reserved  */
/*                                                            */
/**************************************************************/
/*

2006-11-16
  add auxbyte1st as a member of K5VSSP_file variable
2006-09-20
  get_file_size was updated to support large file in case of VC++
2006-01-31
  libvssp32.h includes type definitions and functions 
  for utilities under src directory those use K5/VSSP32 format data.
2007-02-21
  add k5vssp_t_ini function
2007-02-08
  update due to minor change of VSSP32 format as follows
    error flag added, # of bits for year is reduced from 7 to 6.
2007-02-26
  debug in fwd1secK5
2007-03-02  
  more robust sync finding was adopted.
    1) allow 1bit pattern reversal in a sync field
    2) use sampling parameters of 1st frame as sync pattern
    3) use time information to recognize header 
2007-04-17
   consider in case of C++ 
2007-04-24
   libvssp32.h is divided into libvssp32.c and libvssp32.h
2007-10-19
  debug in fwd1seck5 (bug was introduced in 2007-02-06).
  #define ALLOW_USE_SECONDS was commented out
2008-08-08
  "fwdNsecK5" was updated to support forward data exceeding 2GB.
   Previous version was name-changed to "fwdNsecK5org" and called
   from fwdNsecK5.
2009-03-09
   A bug in "fwdNsecK5org" was fixed. This bug causes funny behavior regarding
   nsec parameter; nsec=3 gave the same result as nsec=2, and actual start time
   offset was nsec-1 for nsec >= 3.
2009-07-31
   "k5vssp_t_ini" was updated (K5->headsizew=0 is added)
   "fwd1secK5" was updated to make more robust against errored data
   "fwdNsecK5org" was updated to make more confidential result
2010-08-03
   "CheckK5header_with_Read_but_no_search" was added to support "no header check mode"
   in "k5tom5" and "k5tom5b" 
*/
#ifndef _LIBVSSP32_
#define _LIBVSSP32_

#include "libk5.h"      // common header for IP-VLBI utilities

#define REWIND_BYTE 200 
//#define USE_ASM    // use inline assemble function

/* definition of Robust sync pattern detection */
#define ALLOW_1BIT_REVERSE   // allow one bit reversal in a sync pattern
#define ALLOW_USE_SAMPLE_PARAM  // allow use of sampling parameters
#define ALLOW_USE_SECONDS   // allow time data a sync pattern
#define ALLOW_AUX_MISALIGN   // allow aux field misalignment
             //  right AUX field order  W00, W01, W02, .. W12   (W is 16bit word)
             // misaligned order        W01, W02, .... W12, W00

#ifdef WIN32
  #ifndef USE_ASM
    #define USE_ASM    // use inline assemble function
  #endif
#endif
                              

typedef struct K5VSSP_file     // originated from ip_vlbi_file type
                               // difference is displayed by **K5**
{
    /*
      Note: use of a fpos_t type variable was obsoleted because of
         the compatiblity ploblem of a fpos_t type variable between linux 2.6
         and previous one.
    */
	//unsigned char bit32[4];  // obsoleted in **K5**
	//unsigned char bit64[8]; // obsoleted in **K5**
	unsigned char *ibuf; // will be obsoleted in **K5**
	u_int32_t head32[8]; // **K5**  to contain full header including 
                         // in case of K5/VSSP32
	u_int16_t *head16; // **K5**  pointer to headder in 2 bytes unit  
	unsigned char *head8; // **K5**  pointer to headder in byte unit  
	u_int32_t *aux32; // **K5**  to contain aux field start address in
                        // head 32
	u_int32_t *ubuf; // pointer to read buffer **K5**
    int nword;       // ubuf size allocated or to be allocated  **K5**
    int auxbyte;     // # of bytes in aux field   **K5**
                     // valid byte number in aux32  (deflt=20)
    int auxbyte1st;   // # of bytes in aux field at the 1st headder  **K5**
    int headsizew;    // header size in 32bit-word
    int *idata;   // pointer to decoded data (integer) container
    double *data;   // pointer to decoded data (R*8) container
	int sfreq;  //  sampling frequency in kHz
	int adbit;  // AD bits (1,2,4,8)
	int numch;  // # of channels
	int icnt;   //  current unit number in a second data 
	int numb;   //  # of bytes in unit # of samples
	int usampl;  // unit # of samples
	int kpos;  // should be obsoleted ?
	int npos;    //  valid number in idata or xdata **K5**
	int imax;    //  # of units in a second data
	int bytesec;    //  # of bytes in a second data  **K5**
	//char* fname; //pointer data file name obsoleted
	char fname[128]; //data file name opened or to be opened  **K5**
	double fsample;  // sampling frequency (Hz)
	double tsample; // sampling period (sec)
	double unitp;   // usampl*tsampl unit length 
    long int seconds; // time (sec) from mid night (00h00m00s)
	int hh, mm, ss;  // time in hour, minutes, seconds 
    int eflg; // error flag (K5/VSSP32)  added on 2007-02-07
    int year; // 4 digits year (K5/VSSP32)
    int totalday; // (K5/VSSP32)
    int version1; // Major version (K5/VSSP32)
    int version2; // Minor version (K5/VSSP32)
	int spatn; // detected 2nd sync pattern detected 
	int syncok; // =1 for 32bit all 1 detection // not used
	int format; // =0 for old format , =1 K5/VSSP, =2 K5/VSSP32
                //         =2 is added **K5**
	FILE *stream;
	//fpos_t pos;   // changed to int64_t type 
	//fpos_t size;  // changed to int64_t type
                    // because fpos_t type is no more scalar value in linux 2.6
	int64_t pos;   // current pointer
	int64_t size;  // opened file size in bytes **K5**
	int fh;
	int ch;    
	int ierr;   // error code (general purpose)
	double dtime;  // time at begin of unit
	double dtimest;  // time at begin of file data
	unsigned int bytesread;  // number of bytes read
	int64_t tbytesread;   // total # of bytes read **K5**
	int wordread;   // # of 32bit-words read counter (general) **K5**
	int64_t twordread;   // total # of 32bit-word read **K5**
    int vflag, vflag_old;  // byte slip flag  added on 2003-02-14
                           //  if header sync detection is suceed
                           //     set to 1.  else 0
                           //   1: sync detection success 
                           //   0: sync detection error (may be byte slip)
                           //   vflag_old is the previous status
    long int dtime_vflag, dtime_vflag_old; // time at vflag decision made
	// add 2007-03-02 for robust sync pattern dfinding
	//      sampling parameters and time at 1 st frame
	int sfreq1st;  //  sampling frequency in kHz
	int adbit1st;  // AD bits (1,2,4,8)
	int numch1st;  // # of channels
    long int seconds1st; // time (sec) from mid night (00h00m00s)
    long int secold; // time (sec) from mid night (00h00m00s)
	int framenum;  // frame number starting from 0
} K5VSSP_file_t;

#ifdef __cplusplus
extern "C"{
#endif 

void k5vssp_t_ini(K5VSSP_file_t *K5);
int CheckK5header_with_Read(K5VSSP_file_t *K5);
int64_t get_file_size(char *fname);
void vssp_aux_parse(vsspaux_t *vsspaux);
int CheckK5header(K5VSSP_file_t *K5);
void decode_data(int *K5pos, int wd, int ilast, int pat, int adbit);
void parseADdata_3N(K5VSSP_file_t *K5);
int CheckK5header_with_Read2(K5VSSP_file_t *K5, int maxword);
int fwdNsecK5(int nsec, K5VSSP_file_t *K5);
int fwd1secK5(K5VSSP_file_t *K5);
int fwdNsecK5org(int nsec, K5VSSP_file_t *K5);   // 2008-08-08
int CheckK5header_with_Read_but_no_search(K5VSSP_file_t *S); //2010-08-03

#ifdef __cplusplus
}
#endif

#endif
